import { Component, OnInit } from '@angular/core';
import { ShoutboxDataService } from './services/ShoutboxData.service';
import { makeTestnetWallet } from './util/TestnetUtils';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit {
  lastMessageHash = ""
  contractAddress = "";
  initializing = true
  waiting = false
  title = 'httXrp Shoutbox';
  shouts: any[] = []
  sending = false
  newShout = {
    title: "",
    body: "",
    from: "",
    hash: ""
  }
  userWallet = {
    address: "",
    secret: ""
  }
  drainWallet = {
    address: "",
    secret: ""
  }

  constructor(
    private dataService: ShoutboxDataService
  ) {
    this.contractAddress = dataService.getContractAddress()
  }

  submitShout = () => {
    this.sending = true
    this.dataService.submitShout(this.newShout)
      .then((hash) => {
        this.newShout = {
          title: "",
          body: "",
          from: "",
          hash: ""
        }
        this.lastMessageHash = hash
      })
      .finally(() => {
        this.sending = false
        this.waiting = true
        setTimeout(() => { this.waiting = false }, 20000)
      })
  }

  ngOnInit() {
    Promise.all([makeTestnetWallet(), makeTestnetWallet()]).then(([userWallet, drainWallet]) => {
      console.log(userWallet, drainWallet)
      this.userWallet = userWallet
      this.drainWallet = drainWallet

      this.dataService.initialize((shout: any) => {
        this.shouts.unshift(shout)
        this.initializing = false
      }, userWallet, drainWallet)
    })
  }
}
