import { Injectable } from "@angular/core";
import { z } from 'zod';

declare const RJSVM: any;
declare const RJSVM_Builder: any;
declare const Datawriter: any;

const xrpNode = "wss://s.altnet.rippletest.net:51233"
const listeningAddress = "rMBYWyxGx1b5zEjJKF18TTwF5X3vP2WyjR"

@Injectable()
export class ShoutboxDataService {

    public history: any[] = []
    private dataWriter = undefined as any

    public submitShout = async (shout: any) => {
        return await this.dataWriter.callEndpoint('submit', shout, 10)
    }

    public getContractAddress = () => listeningAddress

    initialize = async (onData:Function, userWallet?:any, drainWallet?:any) => {

        this.dataWriter = new Datawriter({
            receiveAddress: drainWallet.address,
            sendWallet: userWallet,
            xrpNode: xrpNode,
            contractAddress: listeningAddress
        })

        const shoutSchema = z.object({
            title: z.string(),
            body: z.string(),
            from: z.string(),
            id: z.optional(z.string())
        })
        //type Shout = z.infer<typeof shoutSchema>
    
        type State = {
            shouts: any[]
        }
    
        // #########################
        // Define endpoints
        // #########################
    
        type RJSVM_Endpoints = {
            submit: (data: any) => void
        }
    
        // #########################
        // Define init state
        // #########################
    
        abstract class RJSVM_Base
            extends RJSVM<State, RJSVM_Endpoints>
            {
            
            owner = userWallet.address
    
            state: State = {
                shouts: []
            }
        }
    
        // #########################
        // Implement logic
        // #########################
    
        const RJSVM_Contract = {
            submit: {
                implementation: function (env:any, shout: any) {
                    shout.hash = env.hash;
                    (this as any).state.shouts.unshift(shout)
                },
                visibility: 'public',
                fee: 10,
                parameterSchema: shoutSchema
            }
        }
    
        // #########################
        // Build and connect
        // #########################
    
        const Rjsvm = RJSVM_Builder.from(RJSVM_Base, RJSVM_Contract);
    
        const conf = {
            listeningAddress: listeningAddress,
            rippleNode: xrpNode
        }
    
        const rjsvm = new Rjsvm(conf)
        await rjsvm.connect()

        rjsvm.on('error', console.log)
        rjsvm.on('submit', onData)

        this.history = rjsvm.state.shouts
    }
}